--
-- Vyvojova deska s PIC16F887A a XILINX XC7572XL
--	www.prochazka.zde.cz
--
--	VERZE KODU 2.4.3
--
-- Datum : 5.11.2005
--
--	
-- Klavesnice v multiplexnim provozu. Verze programu 2.0. 
-- Rozdil mezi verzi 1.0 a 2.0 je ze nizsi verze dokaze obslouzit 16 tlacitek zapojenych do multiplexniho provozu
--		to znamena ze ma 4. radky a stejny pocet sloupcu. Novejsi verze dokaze obslouzit pouze 8 tlacitek
--		v multiplexnim provozu.	Aplikace vyvojove desky PIC + XILINX nevizadovala takovy velky pocet tlacitek.
--
-- Verze 1.0 je ulozena na www.prochazka.zde.cz na url: http://prochazka.d2.cz/vhdl_cast3.php
-- Klavesnice ma 4. sloupce (S0, S1, S2 a S3) a dva radky (V0 a V1).
--
-- Popis entity klavesnice:
--	  Zakladni signaly:
--		reset --> inicializace modulu klavesnice. Jedna se o fyzicky signal /RESET.
--		clk --> hodiny. Vyvojova deska PIC16F877A + XILINX XC9572XL je taktovana a 20MHz.
--	  Pripojeny ke klavesnice v multiplexnim provozu:
--		sloupce --> to jsou signali S0, S1, S2 a S3.
--		radky --> to jsou signali V0 a V1.
--   Vystupni signal:
--		vystup --> jedna se o cislo, hodnotu ktera reprezentuje stisknute tlacitko. S touto promennou se potom pracuje v celem projektu.
--						
--		Tlacitko A reprezentuje hodnotu v promenne vystup:	 14 binarne "1110"
--		Tlacitko B reprezentuje hodnotu v promenne vystup:	 13 binarne "1101"
--		Tlacitko C reprezentuje hodnotu v promenne vystup:	 12 binarne "1100"
--		Tlacitko D reprezentuje hodnotu v promenne vystup:	 11 binarne "1011"	
--		Tlacitko E reprezentuje hodnotu v promenne vystup:	 10 binarne "1010"
--		Tlacitko F reprezentuje hodnotu v promenne vystup:	 9  binarne "1001"
--		Tlacitko G reprezentuje hodnotu v promenne vystup:	 8  binarne "1000"
--		Tlacitko H reprezentuje hodnotu v promenne vystup:	 7  binarne "0111"
-- Hodnoty jsou negovany. Pokud si prejete jine hodnoty, tak potreba upravit dekodovaci tabulku vystupu VYSTUP.
--
--	Tento modul prevadi informace z multiplexni klavesnice na jednotlive bity, reprezentujici stisknutou klavesu (A az H).
	

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity klavesnice is
    Port ( sloupce : in std_logic_vector(3 downto 0);
           radky : out std_logic_vector(1 downto 0);
           vystup : out std_logic_vector(3 downto 0);
			  reset : in std_logic;
           clk : in std_logic);
end klavesnice;

architecture kl1 of klavesnice is
signal counter : std_logic_vector(2 downto 0);
signal akumulator : std_logic;
signal buffer_counret : std_logic_vector(2 downto 0);
begin

process (clk, reset) 
begin

	if reset = '0' then
--pocatecni inicializace
		counter<="000";
		vystup <="1111";
		buffer_counret <="000";

 	elsif clk = '1' and clk'event then
--cekani na hodinovi impuls	
		counter <= counter +1;

		if akumulator = '0' then  
			buffer_counret <= counter;
--uchovat stav pocitadla pro pozdejsi porovnani, zda je stisknute podad tlacitko
			case	counter is
--zde je nastaveni pozadovane hodnoty na vystupu. Je mozno jakoliv upravovat.
				when "000" => vystup <="1110";	--1
				when "101" => vystup <="1101";	--2
				when "110" => vystup <="1100";	--3 
				when "111" => vystup <="1011";	--4
				when "100" => vystup <="1010";	--5
				when "001" => vystup <="1001";	--6
				when "010" => vystup <="1000";	--7
				when "011" => vystup <="0111";	--8
				when others => NULL;
			end case;
		else
			if	counter =  buffer_counret then
-- pokud se pocitadlo rovna s ulozenim pocitadla, tak uz neni stisknute tlacitka. Kdyby bylo
-- to by se akumulator rovnal nule. Ale tedka se rovna jednicce.
				vystup <="1111"; --Nastavi vsechno na log.1 pokud neni stisknute zadne tlacitko.
			end if;
		end if;
	end if;

	if clk = '0' and clk'event then
--cekat na sestupnou hranu a potom provest kopirovani
		case	counter is
			when "000" => akumulator <= sloupce(0);
							  radky <= "10";
   		when "001" => akumulator <= sloupce(1);
							  radky <= "10";
   		when "010" => akumulator <= sloupce(2);
							  radky <= "10";
			when "011" => akumulator <= sloupce(3);
							  radky <= "10";
			when "100" => akumulator <= sloupce(0);
							  radky <= "01";
   		when "101" => akumulator <= sloupce(1);
							  radky <= "01";
   		when "110" => akumulator <= sloupce(2);
							  radky <= "01";
			when "111" => akumulator <= sloupce(3);
							  radky <= "01";
			when others => NULL;
		end case;
	end if;
end process;
end architecture;

